{% extends 'allianceauth/base.html' %}

{% load i18n %}

{% block page_title %}{% translate "ESI Status" %}{% endblock %}

{% block content %}
    <div class="aa-esi-status">
        <h1 class="page-header text-center">{% translate "ESI Status" %}</h1>
        {% block details %}{% endblock %}
    </div>
{% endblock %}
